@@ -25,6 +25,8 @@ use crate::formatter::Formatter;
 #[cfg(feature = "lipsum")]
 use lipsum::lipsum;
 use lazy_static::lazy_static;
+#[cfg(feature = "cindex")]
+use cindex::OutOption;
 
 lazy_static! {
     static ref CLRF_MATCH: Regex = Regex::new(r#"\r\n"#).unwrap();
@@ -82,6 +84,7 @@ impl BasicMacroMap {
             ("nassert".to_owned(), BMacroSign::new("nassert", ["a_lvalue","a_rvalue"],Self::assert_ne)),
             ("chomp".to_owned(),   BMacroSign::new("chomp",   ["a_content"],Self::chomp)),
             ("comp".to_owned(),    BMacroSign::new("comp",    ["a_content"],Self::compress)),
+            ("dnl".to_owned(),     BMacroSign::new("dnl",     ESR,Self::deny_newline)),
             ("env".to_owned(),     BMacroSign::new("env",     ["a_env_name"],Self::get_env)),
             ("envset".to_owned(),  BMacroSign::new("envset",  ["a_env_name","a_env_value"],Self::set_env)),
             ("escape".to_owned(),  BMacroSign::new("escape",  ESR,Self::escape)),
@@ -96,6 +99,7 @@ impl BasicMacroMap {
             ("panic".to_owned(),   BMacroSign::new("panic",   ["a_msg"],Self::manual_panic)),
             ("path".to_owned(),    BMacroSign::new("path",    ["a_paths"],Self::merge_path)),
             ("pipe".to_owned(),    BMacroSign::new("pipe",    ["a_value"],Self::pipe)),
+            ("pipeto".to_owned(),  BMacroSign::new("pipe",    ["a_pipe_name","a_value"],Self::pipe_to)),
             ("read".to_owned(),    BMacroSign::new("read",    ["a_filename"],Self::read)),
             ("redir".to_owned(),   BMacroSign::new("redir",   ["a_redirect?"],Self::temp_redirect)),
             ("regex".to_owned(),   BMacroSign::new("regex",   ["a_source","a_match","a_substitution"],Self::regex_sub)),
@@ -120,6 +124,12 @@ impl BasicMacroMap {
             map.insert("from".to_owned(),    BMacroSign::new("from", ["a_macro_name","a_csv_value"],Self::from_data));
             map.insert("table".to_owned(),   BMacroSign::new("table",["a_table_form","a_csv_value"],Self::table));
         }
+        #[cfg(feature = "cindex")]
+        {
+            map.insert("regcsv".to_owned(),  BMacroSign::new("regcsv", ["a_table_name","a_table"], Self::cindex_register));
+            map.insert("query".to_owned(),   BMacroSign::new("query",  ["a_query"], Self::cindex_query));
+        }
+
         #[cfg(feature = "chrono")]
         {
             map.insert("time".to_owned(),    BMacroSign::new("time",ESR,Self::time));
@@ -138,6 +148,12 @@ impl BasicMacroMap {
             map.insert("hookoff".to_owned(), BMacroSign::new("hookoff",["a_macro_type","a_target_name"],Self::hook_disable));
         }
 
+        #[cfg(feature = "storage")]
+        {
+            map.insert("update".to_owned(),      BMacroSign::new("update",  ["a_text"],Self::update_storage));
+            map.insert("extract".to_owned(),     BMacroSign::new("extract", ESR,Self::extract_storage));
+        }
+
         // Return struct
         Self { macros : map }
     }
@@ -680,7 +696,23 @@ impl BasicMacroMap {
     /// $pipe(Value)
     fn pipe(args: &str, greedy: bool, processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
-            processor.state.pipe_value = args[0].to_owned();
+            processor.state.add_pipe(None, args[0].to_owned());
+        }
+        Ok(None)
+    }
+
+    /// Put value into a temporary stack called pipe
+    ///
+    /// Piped value can be popped with macro '-'
+    ///
+    /// # Usage
+    ///
+    /// $pipeto(Value)
+    fn pipe_to(args: &str, greedy: bool, processor: &mut Processor) -> RadResult<Option<String>> {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
+            processor.state.add_pipe(Some(&args[0]), args[1].to_owned());
+        } else {
+            return Err(RadError::InvalidArgument("pipeto requires two arguments".to_owned()));
         }
         Ok(None)
     }
@@ -775,6 +807,16 @@ impl BasicMacroMap {
     fn newline(_: &str, _: bool, p: &mut Processor) -> RadResult<Option<String>> {
         Ok(Some(p.state.newline.to_owned()))
     }
+    
+    /// deny new line
+    ///
+    /// # Usage
+    ///
+    /// $dnl()
+    fn deny_newline(_: &str, _: bool, p: &mut Processor) -> RadResult<Option<String>> {
+        p.state.deny_newline = true;
+        Ok(None)
+    }
 
     /// Get name from given path
     ///
@@ -842,9 +884,26 @@ impl BasicMacroMap {
     /// # Usage
     ///
     /// $-()
-    fn get_pipe(_: &str, _: bool, processor: &mut Processor) -> RadResult<Option<String>> {
-        let out = processor.state.pipe_value.clone();
-        Ok(Some(out))
+    /// $-(p1)
+    fn get_pipe(args: &str, greedy: bool, processor: &mut Processor) -> RadResult<Option<String>> {
+        let pipe = if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
+            let name = Utils::trim(&args[0]);
+            if name.is_empty() {
+                let out = processor.state.get_pipe("-").unwrap_or(String::new()).clone();
+                Some(out)
+            } else {
+                if let Some(pipe) = processor.state.get_pipe(&args[0]) {
+                    Some(pipe.clone())
+                } else {
+                    None
+                }
+            }
+        } else {
+            // "-" Always exsit, thus safe to unwrap
+            let out = processor.state.get_pipe("-").unwrap_or(String::new()).clone();
+            Some(out)
+        };
+        Ok(pipe)
     }
 
     /// Return a length of the string
@@ -1066,7 +1125,7 @@ impl BasicMacroMap {
             return Ok(None);
         }
         if let Some(args) = ArgParser::new().args_with_len(args, 1, false) {
-            let toggle = if let Ok(toggle) =Utils::is_arg_true(&args[0]) { 
+            let toggle = if let Ok(toggle) = Utils::is_arg_true(&args[0]) { 
                 toggle
             } else {
                 return Err(RadError::InvalidArgument(format!("Redir's agument should be valid boolean value but given \"{}\"", &args[0])));
@@ -1145,6 +1204,57 @@ impl BasicMacroMap {
             Err(RadError::InvalidArgument("Wrap requires two arguments".to_owned()))
         }
     }
+
+    #[cfg(feature = "storage")]
+    fn update_storage(args: &str, _: bool, processor: &mut Processor) -> RadResult<Option<String>> {
+        let args = ArgParser::new().args_to_vec(args, ',', GreedyState::Never);
+
+        // Execute update method for storage
+        if let Some(storage) = processor.storage.as_mut() {
+            if let Err(err) = storage.update(&args) {
+                return Err(RadError::StorageError(format!("Update error : {}", err)));
+            }
+        } else { 
+            processor.log_warning("Empty storage, update didn't triggerd")?;
+        }
+        Ok(None)
+    }
+
+    #[cfg(feature = "storage")]
+    fn extract_storage(_: &str, _: bool, processor: &mut Processor) -> RadResult<Option<String>> {
+        // Execute update method for storage
+        if let Some(storage) = processor.storage.as_mut() {
+            match storage.extract(false) {
+                Err(err) => Err(RadError::StorageError(format!("Update error : {}", err))),
+                Ok(value) => {
+                    if let Some(output) = value {
+                        Ok(Some(output.into_printable()))
+                    } else { Ok(None) }
+                },
+            }
+        } else { Err(RadError::StorageError(String::from("Empty storage"))) }
+    }
+
+    #[cfg(feature = "cindex")]
+    fn cindex_register(args: &str, _: bool, processor: &mut Processor) -> RadResult<Option<String>> {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, true) {
+            processor.indexer.add_table_fast(&args[0], args[1].as_bytes())?;
+            Ok(None)
+        } else {
+            Err(RadError::InvalidArgument("regcsv requires two arguments".to_owned()))
+        }
+    }
+
+    #[cfg(feature = "cindex")]
+    fn cindex_query(args: &str, _: bool, processor: &mut Processor) -> RadResult<Option<String>> {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, true) {
+            let mut value = String::new();
+            processor.indexer.index_raw(&Utils::trim(&args[0]), OutOption::Value(&mut value))?;
+            Ok(Some(Utils::trim(&value)))
+        } else {
+            Err(RadError::InvalidArgument("query requires an argument".to_owned()))
+        }
+    }
 }
 
 // TODO
