@@ -3,6 +3,7 @@
 //!
 //! Cli module is only included in binary feature flag.
 
+use std::io::Read;
 use crate::RadResult;
 use crate::auth::AuthType;
 use crate::processor::Processor;
@@ -95,16 +96,33 @@ impl Cli {
         processor.print_permission()?;
         
         // Process
+        // Redirect stdin as argument
+        if args.is_present("pipe") {
+            let stdin = std::io::stdin();
+            let mut input = String::new();
+            stdin.lock().read_to_string(&mut input)?;
+            processor.set_pipe(&input)
+        }
+        
         // -->> Read from files
-        if let Some(files) = args.values_of("FILE") {
+        if let Some(sources) = args.values_of("INPUT") {
             // Also read from stdin if given combiation option
             if args.is_present("combination") {
                 processor.from_stdin()?;
             }
 
-            // Read from files and write with given options
-            for file in files {
-                processor.from_file(Path::new(file))?;
+            // Interpret every input source as literal text
+            let literal = args.is_present("literal");
+
+            // Read from given sources and write with given options
+            for src in sources {
+                let src_as_file = Path::new(src);
+
+                if !literal && src_as_file.exists() {
+                    processor.from_file(src_as_file)?;
+                } else {
+                    processor.from_string(src)?;
+                }
             }
             #[cfg(feature = "signature")]
             self.print_signature(args, &mut processor)?;
@@ -212,7 +230,7 @@ impl Cli {
     fn args_builder() -> clap::ArgMatches {
         use clap::{App,Arg};
         let app = App::new("rad")
-            .version("1.4.1")
+            .version("1.5.0")
             .author("Simon creek <simoncreek@tutanota.com>")
             .about( "R4d(rad) is a modern macro processor made with rust. Refer https://github.com/simhyeon/r4d for detailed usage.")
             .long_about("R4d is a text oriented macro processor which aims to be an modern alternative to m4 macro processor. R4d procedurally follows texts and substitue macro calls with defined macro body. R4d comes with varoius useful built in macros so that user don't have to define from scratch. R4d also supports multiple debugging flags for easier error detection. Refer https://github.com/simhyeon/r4d for detailed usage.")
@@ -222,123 +240,130 @@ impl Cli {
     echo <STDIN_TEXT> | rad --combination <FILE> --diff
     rad <FILE> --debug --log --interactive
     rad <FILE> -f <RULE_FILE> --discard -n --silent")
-            .arg(Arg::new("FILE")
+            .arg(Arg::new("INPUT")
                 .multiple_values(true)
-                .about("Files to execute processing"))
+                .help("INPUT source to execute processing"))
+            .arg(Arg::new("pipe")
+                .long("pipe")
+                .conflicts_with("combination")
+                .help("Send stdin as a pipe value"))
+            .arg(Arg::new("literal")
+                .long("literal")
+                .help("Don't interpret input source as file"))
             .arg(Arg::new("out")
                 .short('o')
                 .long("out")
                 .takes_value(true)
                 .conflicts_with("discard")
                 .value_name("FILE")
-                .about("Save processed output to the file"))
+                .help("Save processed output to the file"))
             .arg(Arg::new("err")
                 .short('e')
                 .long("err")
                 .takes_value(true)
                 .value_name("FILE")
-                .about("Save error logs to the file"))
+                .help("Save error logs to the file"))
             .arg(Arg::new("combination")
                 .short('c')
                 .long("combination")
-                .about("Read from both stdin and file inputs. Stdin is evaluated first"))
+                .help("Read from both stdin and file inputs. Stdin is evaluated first"))
             .arg(Arg::new("discard")
                 .short('D')
                 .long("discard")
-                .about("Discard output"))
+                .help("Discard output"))
             .arg(Arg::new("silent")
                 .short('s')
                 .long("silent")
-                .about("Supress warnings"))
+                .help("Supress warnings"))
             .arg(Arg::new("greedy")
                 .short('g')
                 .long("greedy")
-                .about("Make all macro invocations greedy"))
+                .help("Make all macro invocations greedy"))
             .arg(Arg::new("purge")
                 .short('p')
                 .long("purge")
-                .about("Purge unused macros without panicking. Doesn't work in strict mode"))
+                .help("Purge unused macros without panicking. Doesn't work in strict mode"))
             .arg(Arg::new("lenient")
                 .short('l')
-                .long("long")
-                .about("Lenient mode, disables strict mode"))
+                .long("lenient")
+                .help("Lenient mode, disables strict mode"))
             .arg(Arg::new("nopanic")
                 .long("nopanic")
-                .about("Don't panic in any circumstances, the most lenient mode"))
+                .help("Don't panic in any circumstances, the most lenient mode"))
             .arg(Arg::new("debug")
                 .short('d')
                 .long("debug")
-                .about("Debug mode"))
+                .help("Debug mode"))
             .arg(Arg::new("log")
                 .long("log")
-                .about("Print log for every macro invocation. Only works on debug mode"))
+                .help("Print log for every macro invocation. Only works on debug mode"))
             .arg(Arg::new("diff")
                 .long("diff")
                 .takes_value(true)
                 .value_name("DIFF TYPE")
-                .default_value("all")
-                .about("Show diff result (none|change|all)"))
+                .default_missing_value("all")
+                .help("Show diff result (none|change|all)"))
             .arg(Arg::new("interactive")
                 .short('i')
                 .long("interactive")
-                .about("Use interactive debug mode. This enables line wrapping."))
+                .help("Use interactive debug mode. This enables line wrapping."))
             .arg(Arg::new("assert")
                 .long("assert")
-                .about("Enable assert mode"))
+                .help("Enable assert mode"))
             .arg(Arg::new("comment")
                 .long("comment")
                 .takes_value(true)
-                .default_value("start")
+                .default_missing_value("start")
                 .value_name("COMMENT TYPE")
-                .about("Use comment option (none|start|any)"))
+                .help("Use comment option (none|start|any)"))
             .arg(Arg::new("allow")
                 .short('a')
                 .takes_value(true)
                 .value_name("AUTH TYPE")
-                .about("Allow permission (fin|fout|cmd|env)"))
+                .help("Allow permission (fin|fout|cmd|env)"))
             .arg(Arg::new("allow_warn")
                 .short('w')
                 .takes_value(true)
                 .value_name("AUTH TYPE")
-                .about("Allow permission with warnings (fin|fout|cmd|env)"))
+                .help("Allow permission with warnings (fin|fout|cmd|env)"))
             .arg(Arg::new("allow_all")
                 .short('A')
                 .conflicts_with("allow_all_warn")
-                .about("Allow all permission"))
+                .help("Allow all permission"))
             .arg(Arg::new("allow_all_warn")
                 .short('W')
                 .conflicts_with("allow_all")
-                .about("Allow all permission with warning"))
+                .help("Allow all permission with warning"))
             .arg(Arg::new("newline")
                 .short('n')
                 .long("newline")
-                .about("Use unix newline for formatting"))
+                .help("Use unix newline for formatting"))
             .arg(Arg::new("melt")
                 .short('m')
                 .long("melt")
                 .takes_value(true)
                 .value_name("FILE")
-                .about("Read macros from frozen file"))
+                .help("Read macros from frozen file"))
             .arg(Arg::new("freeze")
                 .short('f')
                 .long("freeze")
                 .takes_value(true)
                 .value_name("FILE")
-                .about("Freeze macros into a single file"));
+                .help("Freeze macros into a single file"));
 
         #[cfg(feature = "signature")]
         let app = app.arg(Arg::new("signature")
                 .long("signature")
                 .takes_value(true)
                 .value_name("FILE")
-                .default_value(" ")
-                .about("Print signature to file."))
+                .default_missing_value(" ")
+                .help("Print signature to file."))
             .arg(Arg::new("sigtype")
                 .long("sigtype")
                 .takes_value(true)
                 .value_name("SIG TYPE")
                 .default_value("all")
-                .about("Signature type to get"));
+                .help("Signature type to get"));
 
         app.get_matches()
     }
