@@ -1,3 +1,7 @@
+#[cfg(feature = "cindex")]
+use cindex::CIndexError;
+#[cfg(feature = "csv")]
+use csv::FromUtf8Error;
 use thiserror::Error;
 use crate::AuthType;
 
@@ -14,6 +18,12 @@ pub enum RadError {
     UnallowedChar(String),
     #[error("Assert failed")]
     AssertFail,
+    #[cfg(feature = "csv")]
+    #[error("Invalid byte array conversion to string")]
+    InvalidString(FromUtf8Error),
+    #[cfg(not(feature = "csv"))]
+    #[error("Invalid conversion to string")]
+    InvalidString(std::str::Utf8Error),
     #[error("Invalid command option\n= {0}")]
     InvalidCommandOption(String),
     #[error("Invalid environment name\n= {0}")]
@@ -51,7 +61,13 @@ pub enum RadError {
     #[error("Processor panicked, exiting...")]
     Panic,
     #[error("Panic triggered with message\n= {0}")]
-    ManualPanic(String)
+    ManualPanic(String),
+    #[cfg(feature = "storage")]
+    #[error("Storage error with message\n= {0}")]
+    StorageError(String),
+    #[cfg(feature = "cindex")]
+    #[error("{0}")]
+    CIndexError(CIndexError),
 }
 
 // ==========
@@ -106,6 +122,20 @@ impl From <std::env::VarError> for RadError {
         Self::EnvError(err)
     }
 }
+
+#[cfg(feature = "csv")]
+impl From <FromUtf8Error> for RadError {
+    fn from(err : FromUtf8Error) -> Self {
+        Self::InvalidString(err)
+    }
+}
+
+#[cfg(feature = "cindex")]
+impl From <CIndexError> for RadError {
+    fn from(err : CIndexError) -> Self {
+        Self::CIndexError(err)
+    }
+}
 // End of convert variations
 // </CONVERT>
 // ----------
