@@ -11,8 +11,9 @@ use bincode;
 pub type RadResult<T> = Result<T, RadError>;
 
 /// State enum value about direction of processed text 
-pub enum WriteOption {
+pub enum WriteOption<'a> {
     File(std::fs::File),
+    Variable(&'a mut String),
     Terminal,
     Discard,
 }
@@ -290,6 +291,16 @@ impl RuleFile {
         }
     }
 
+    pub fn melt_literal(&mut self, literal : &Vec<u8>) -> RadResult<()> {
+        let result = bincode::deserialize::<Self>(literal);
+        if let Err(_) = result {
+            Err(RadError::BincodeError(format!("Failed to melt the literal value")))
+        } else {
+            self.rules.extend(result.unwrap().rules.into_iter());
+            Ok(())
+        }
+    }
+
     /// Convert custom rules into a single binary file
     pub(crate) fn freeze(&self, path: &std::path::Path) -> RadResult<()> {
         let result = bincode::serialize(self);
@@ -444,3 +455,29 @@ impl SignatureType {
         Ok(variant)
     }
 }
+
+#[cfg(feature = "storage")]
+pub type StorageResult<T> = Result<T, Box<dyn std::error::Error>>;
+
+#[cfg(feature = "storage")]
+pub trait RadStorage {
+    fn update(&mut self,args : &Vec<String>) -> StorageResult<()>;
+    fn extract(&mut self, serialize: bool) -> StorageResult<Option<StorageOutput>>;
+}
+
+#[cfg(feature = "storage")]
+#[derive(Debug)]
+pub enum StorageOutput {
+    Binary(Vec<u8>),
+    Text(String),
+}
+
+#[cfg(feature = "storage")]
+impl StorageOutput {
+    pub(crate) fn into_printable(&self) -> String {
+        match self {
+            Self::Binary(bytes) => format!("{:?}", bytes),
+            Self::Text(text) => text.to_owned(),
+        }
+    }
+}
