@@ -119,6 +119,8 @@ use crate::logger::{Logger, LoggerLines};
 #[cfg(feature = "debug")]
 use crate::debugger::Debugger;
 use crate::models::{CommentType, MacroFragment, MacroMap, CustomMacro, RuleFile, UnbalancedChecker, WriteOption, LocalMacro, FlowControl};
+#[cfg(feature = "storage")]
+use crate::models::{RadStorage, StorageOutput};
 #[cfg(feature = "signature")]
 use crate::models::SignatureType;
 #[cfg(feature = "hook")]
@@ -131,6 +133,8 @@ use crate::lexor::*;
 use crate::define_parser::DefineParser;
 use crate::arg_parser::{ArgParser, GreedyState};
 use crate::MacroType;
+#[cfg(feature = "cindex")]
+use cindex::Indexer;
 use regex::Regex;
 use lazy_static::lazy_static;
 
@@ -160,7 +164,8 @@ pub(crate) struct ProcessorState {
     pub newline: String,
     pub nopanic: bool,
     pub paused: bool,
-    pub pipe_value: String,
+    pub pipe_truncate: bool,
+    pipe_map: HashMap<String,String>,
     pub purge: bool,
     pub redirect: bool,
     pub sandbox: bool,
@@ -173,6 +178,7 @@ pub(crate) struct ProcessorState {
     pub comment_char : Option<char>,
     pub macro_char : Option<char>,
     pub flow_control : FlowControl,
+    pub deny_newline : bool,
 }
 
 impl ProcessorState {
@@ -181,7 +187,8 @@ impl ProcessorState {
             current_input: String::from("stdin"),
             auth_flags: AuthFlags::new(),
             newline : LINE_ENDING.to_owned(),
-            pipe_value: String::new(),
+            pipe_truncate: false,
+            pipe_map: HashMap::new(),
             paused: false,
             redirect: false,
             purge: false,
@@ -194,26 +201,47 @@ impl ProcessorState {
             comment_char: None,
             macro_char: None,
             flow_control: FlowControl::None,
+            deny_newline: false,
+        }
+    }
+
+    pub fn add_pipe(&mut self, name: Option<&str>, value: String) {
+        if let Some(name) = name {
+            self.pipe_map.insert(name.to_owned(), value);
+        } else {
+            self.pipe_map.insert("-".to_owned(), value);
+        }
+    }
+
+    pub fn get_pipe(&mut self, key: &str) -> Option<String> {
+        if self.pipe_truncate{
+            self.pipe_map.remove(key)
+        }  else {
+            self.pipe_map.get(key).map(|s| s.to_owned())
         }
     }
 }
 
 /// Processor that parses(lexes) given input and print out to desginated output
-pub struct Processor{
+pub struct Processor<'processor>{
     map: MacroMap,
     closure_map: ClosureMap,
     #[cfg(feature = "hook")]
     pub(crate) hook_map: HookMap,
     defparser: DefineParser,
-    write_option: WriteOption,
-    logger: Logger,
+    write_option: WriteOption<'processor>,
+    logger: Logger<'processor>,
     #[cfg(feature = "debug")]
     debugger: Debugger,
     checker: UnbalancedChecker,
     pub(crate) state: ProcessorState,
+    #[cfg(feature = "storage")]
+    pub storage : Option<Box<dyn RadStorage>>,
+    #[cfg(feature = "cindex")]
+    pub indexer : Indexer
 }
 
-impl Processor {
+impl<'processor> Processor<'processor> {
     // ----------
     // Builder pattern methods
     // <BUILDER>
@@ -271,6 +299,10 @@ impl Processor {
             debugger : Debugger::new(),
             checker : UnbalancedChecker::new(),
             state: ProcessorState::new(temp_target),
+            #[cfg(feature = "storage")]
+            storage : None,
+            #[cfg(feature = "cindex")]
+            indexer: Indexer::new(),
         }
     }
 
@@ -292,6 +324,12 @@ impl Processor {
         Ok(self)
     }
 
+    /// Write to variable
+    pub fn write_to_variable(mut self, value: &'processor mut String) -> Self {
+        self.write_option = WriteOption::Variable(value);
+        self
+    }
+
     /// Yield error to the file
     pub fn error_to_file(mut self, target_file: Option<impl AsRef<Path>>) -> RadResult<Self> {
         if let Some(target_file) = target_file {
@@ -311,6 +349,12 @@ impl Processor {
         Ok(self)
     }
 
+    /// Yield error to the file
+    pub fn error_to_variable(mut self, value: &'processor mut String) -> Self {
+        self.logger.set_write_options(Some(WriteOption::Variable(value)));
+        self
+    }
+
     /// Custom comment character
     pub fn custom_comment_char(mut self, character: char) -> RadResult<Self> {
         // check if unallowed character
@@ -362,6 +406,12 @@ impl Processor {
         self
     }
 
+    /// Set truncate option
+    pub fn pipe_truncate(mut self, truncate: bool) -> Self {
+        self.state.pipe_truncate = truncate;
+        self
+    }
+
     /// Use comment
     ///
     /// This is deprecated and will be removed in 2.0 version.
@@ -434,7 +484,31 @@ impl Processor {
         self
     }
 
+    /// Melt rule file
+    pub fn rule_files(mut self, paths: Option<Vec<impl AsRef<Path>>>) -> RadResult<Self> {
+        if let Some(paths) = paths {
+            let mut rule_file = RuleFile::new(None);
+            for p in paths.iter() {
+                // File validity is checked by melt methods
+                rule_file.melt(p.as_ref())?;
+            }
+            self.map.custom.extend(rule_file.rules);
+        }
+
+        Ok(self)
+    }
+
+    /// Melt rule as literal input source
+    pub fn rule_literal(mut self, literal: &Vec<u8>) -> RadResult<Self> {
+        let mut rule_file = RuleFile::new(None);
+        rule_file.melt_literal(literal)?;
+        self.map.custom.extend(rule_file.rules);
+        Ok(self)
+    }
+
+    // NOTE : Renamed to rule_files
     /// Add custom rules
+    #[deprecated(since = "1.5", note = "Use \"rule_files\" instead.This method will be removed in 2.0")]
     pub fn custom_rules(mut self, paths: Option<Vec<impl AsRef<Path>>>) -> RadResult<Self> {
         if let Some(paths) = paths {
             let mut rule_file = RuleFile::new(None);
@@ -476,6 +550,13 @@ impl Processor {
         self
     }
 
+    /// Build with storage
+    #[cfg(feature = "storage")]
+    pub fn storage(mut self, storage: Box<dyn RadStorage>) -> Self {
+        self.storage.replace(storage);
+        self
+    }
+
     /// Creates an unreferenced instance of processor
     #[deprecated(since = "1.3", note = "Build method is deprecated in favor of more ergonomic builder pattern. It will be removed in 2.0")]
     pub fn build(self) -> Self {
@@ -524,7 +605,26 @@ impl Processor {
         Ok(())
     }
 
-    /// Freeze to single file
+    /// Set storage
+    #[cfg(feature = "storage")]
+    pub fn set_storage(&mut self, storage: Box<dyn RadStorage>) {
+        self.storage.replace(storage);
+    }
+
+    /// Extract from storage
+    #[cfg(feature = "storage")]
+    pub fn extract_storage(&mut self, truncate: bool) -> RadResult<Option<StorageOutput>> {
+        if let Some(storage) = self.storage.as_mut() {
+            match storage.extract(truncate) {
+                Err(err) => Err(RadError::StorageError(format!("Extract error : {}", err))),
+                Ok(value) => Ok(value),
+            }
+        } else {
+            Ok(None)
+        }
+    }
+
+    /// Freeze to a single file
     ///
     /// Frozen file is a bincode encoded binary format file.
     pub fn freeze_to_file(&self, path: impl AsRef<Path>) -> RadResult<()> {
@@ -855,6 +955,14 @@ impl Processor {
         if let Some(line) = lines.next() {
             let line = line?;
 
+            // Deny newline
+            if self.state.deny_newline { 
+                self.state.deny_newline = false;
+                if line == "\n" || line == "\r\n" {
+                    return Ok(ParseResult::NoPrint);
+                }
+            }
+
             match self.state.flow_control {
                 FlowControl::Escape => return Ok(ParseResult::Printable(line)),
                 FlowControl::Exit => return Err(RadError::Exit),
@@ -1182,6 +1290,7 @@ impl Processor {
             match &mut self.write_option {
                 WriteOption::File(f) => f.write_all(content.as_bytes())?,
                 WriteOption::Terminal => print!("{}", content),
+                WriteOption::Variable(var) => var.push_str(content),
                 WriteOption::Discard => () // Don't print anything
             }
         }
@@ -1431,7 +1540,7 @@ impl Processor {
                     // because pipe should respect all other macro attributes 
                     // not the other way
                     if frag.pipe {
-                        self.state.pipe_value = content;
+                        self.state.add_pipe(None, content);
                         lexor.escape_next_newline();
                     } else {
                         remainder.push_str(&content);
@@ -1519,6 +1628,11 @@ impl Processor {
             .unwrap());
     }
 
+    /// Set pipe value manually
+    pub(crate) fn set_pipe(&mut self, value: &str) {
+        self.state.pipe_map.insert("-".to_owned(), value.to_string());
+    }
+
     /// Set debug flag
     #[cfg(feature = "debug")]
     pub(crate) fn set_debug(&mut self, debug: bool)  {
